/***************************************************************************
 *   Copyright (C) 2010 by user,,,   *
 *   user@debian   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "parray.h"

struct parray *
parray_new(size_t init_size)
{
	struct parray *pa;
	int size;

	pa = malloc(sizeof(*pa));

	if (pa == NULL) {
		fprintf(stderr, "can't allocate parray\n");
		return NULL;
	}

	size = sizeof(void *) * (init_size != 0 ? init_size : 32);

	pa->data = malloc(size);

	if (pa->data == NULL) {
		fprintf(stderr, "can't allocate parray data\n");
		free(pa);
		return NULL;
	}

	memset(pa->data, 0, size);

	pa->nalloc = (init_size != 0 ? init_size : 32);
	pa->nelems = 0;

	return pa;
}

int
parray_add(struct parray *pa, void *ptr)
{
	unsigned int ret;

	if (pa->nalloc == pa->nelems) {
		void *ptr;
		size_t new_size, old_size;

		new_size = (pa->nalloc * 2) * sizeof(void *);
		old_size = pa->nalloc * sizeof(void *);

		ptr = realloc(pa->data, new_size);

		if (ptr == NULL) {
			fprintf(stderr, "can't realloc parray data\n");
			return -1;
		}

		memset(pa->data + pa->nalloc, 0, new_size - old_size);
		pa->nalloc *= 2;
		pa->data = ptr;
	}

	ret = pa->nelems;
	pa->data[pa->nelems] = ptr;
	++pa->nelems;

	return ret;
}

int
parray_delete_nth(struct parray *pa, unsigned int nth)
{
	if (nth >= pa->nelems){
		fprintf(stderr, "elems cann't delete\n");
		return -1;
	}
	if (nth < pa->nelems-1) {
		memmove(pa->data+nth, pa->data+nth+1, (pa->nelems-nth+1)*sizeof(void *));
		--pa->nelems;
	} else {
		--pa->nelems;
	}
	return 0;
}

void
parray_delete(struct parray *pa, void *ptr)
{	int i;
	for(i = 0; i < pa->nelems; ++i) {
		if (pa->data[i] == ptr) {
			break;
		}
	}
	if (i < pa->nelems) {
		--pa->nelems;
		if (i < pa->nelems) {
			memmove(pa->data + i, pa->data + i + 1, (pa->nelems - i + 1)*sizeof(void *));
		} else {
			fprintf(stderr, "elems cann't delete\n");
		}
	}

}

void
parray_change(struct parray *pa, void *ptr, unsigned int nth)
{
	if (nth >= pa->nelems) {
		fprintf(stderr, "elems cann't change\n");
		return;
	}
	pa->data[nth] = ptr;
}

void
parray_delete_range(struct parray *pa, unsigned int nth, size_t count)
{
	if (nth >= pa->nelems) {
		fprintf(stderr, "cann't delete range\n");
		return;
	}
	if (count >= pa->nelems-nth) {
		memmove(pa->data+nth, pa->data+pa->nelems, ((pa->nelems -1- nth))*sizeof(void *));
		pa->nelems = pa->nelems - (pa->nelems - nth);
	} else {
		memmove(pa->data+nth, pa->data+nth+count, (pa->nelems -(count - nth))*sizeof(void *));
		pa->nelems = pa->nelems - (count - nth) - 1;
	}
}

void
*parray_get_nth(struct parray *pa, unsigned int nth)
{
	if (nth>pa->nelems) {
		fprintf(stderr, "cann't get pointer\n");
		return;
	}
	return pa->data[nth];
}

struct user_data {
	int age;
};

int
compare(const void *a, const void *b)
{
	struct user_data *ap, *bp;
	ap = (struct user_data *)a;
	bp = (struct user_data *)b;
	if (ap->age < bp->age)
		return -1;
	else if (ap->age > bp->age)
		return 1;
	return 0;
}

int
parray_search(struct parray *pa, void *ptr, parray_compare_t compare_func)
{
	int i, res;

	for(i = 0; i < pa->nelems; i++) {
		res = (*compare_func)(ptr, pa->data[i]);
		if (res == 0)
			return i;
	}

	return -1;
}

void
parray_sort(struct parray *pa, parray_compare_t compare_func)
{
	void *tmp;
	int i, res, j;
	for(i = 0; i <= pa->nelems-1; i++ ) {
		for(j = 0; j <= (pa->nelems - 1) - i; j++) {
			res = compare(pa->data[j], pa->data[j+1]);
			if (res == 1) {
				tmp = pa->data[j];
				pa->data[j] = pa->data[j+1];
				pa->data[j+1] = tmp;
			}
		}
	}
	return 0;
}

int
callback(void *ptr, void *opaque)
{
        struct user_data *up = (struct user_data *) ptr;
        int *flp = (int *) opaque;
        printf("%f %d\n", *flp, up->age)
        return 0;
}

void
*parray_foreach(struct parray *pa, parray_callback_t callback, void *opaque)
{       
        int i, tmp;
        for(i = 0; i <= pa->nelems-1; i++ ) {
        tmp=callback(pa->data[i], opaque);
        if tmp != 0 then return -1;
        }
}


